function Geosetter(map)
{
  this.map = map;
  this.signals = this.map.getSignals();

  this.markers = {};
  this.gsBasePath = "";

  this._initZoomCenterSync();
  this._initBlueMarkers();
  this._initFocusMarker();
  this._initDragMarker();
  this._initTrack();
  this._initPOV();
  this._initMarkerCards();
  this._initFavorites();
  this._initCrosshair();
  this._initCustomLayers();
}

Geosetter._encapsulate = function(functionName)
{
  encapsulate(window, functionName, function()
  {
    var ret = window[functionName + "Orig"].apply(this, arguments);
    
    if (window.gs)
      gs[functionName].apply(gs, arguments);
      
    return ret;    
  });
}

//********************************************************************************
//                            zoom, center, bounds ...
//********************************************************************************

Geosetter.prototype._initZoomCenterSync = function()
{
  JAK.Events.addListener(document.documentElement, "keydown", this, "_mapSwitch", true);
  
  // při každém překreslení mapy přenastavíme Google Maps, aby se ukládala historie
  var that = this;
  
  this.signals.addListener(window, "map-redraw", function(e, elm)
  {
    that._syncGoogleMaps();  
    document.getElementById("changed").innerHTML = "1";
    UpdateValues();
  });  
}

// synchronizuje střed a zoom u Google Maps (zatímco máme zapnuté Mapy.cz)
Geosetter.prototype._syncGoogleMaps = function()
{
  // je třeba volat ještě jednou před, protože pak by byla po prvním přepnutí na Google Maps špatná poloha
  google.maps.event.trigger(map, 'resize');
  
  map.setCenter(this.map.getCenter().toGoogle());
  var nearestZoom = this.map.getNearestZoom({layerTo: "GSatellite_ID"});
  map.setZoom(nearestZoom);
  google.maps.event.trigger(map, 'resize');
}

// synchronizuje střed a zoom u Mapy.cz (zatímco máme zapnuté Google Maps)
Geosetter.prototype._syncMapyCZ = function()
{
  this.map.setCenter(SMap.Coords.fromGoogle(map.getCenter()));
  var nearestZoom = this.map.getNearestZoom({layerFrom: "GSatellite_ID", zoomFrom: map.getZoom()});
  this.map.setZoom(nearestZoom);
  this.map.syncPort();
}

// stiskem středníku se můžeme přepínat mezi oběma verzemi mapy
Geosetter.prototype._mapSwitch = function(e, elm)
{
  if (contains_class(document.body, "lightbox_on"))
    return;

  // středník, bez SHIFTu
  if ((e.keyCode != 192 && e.keyCode != 59 /* Firefox */) || e.shiftKey)
    return;
  
  // změníme která mapa je aktuální  
  toggle_class(document.body, "geosetter_original_map");

  // přepneme se na Google Maps
  if (contains_class(document.body, "geosetter_original_map"))
    this._syncGoogleMaps();

  // přepneme se na Mapy.cz
  else
    this._syncMapyCZ();
}

Geosetter.prototype.zoomBounds = function(ABounds, AMaxZoomLevel)
{
  var ne = SMap.Coords.fromGoogle(ABounds.getNorthEast());
  var sw = SMap.Coords.fromGoogle(ABounds.getSouthWest());

  this.map.setCenterZoom.apply(this.map, this.map.computeCenterZoom([ne, sw]));
}

Geosetter.prototype.setCenter = function(ALatitude, ALongitude)
{
  this.map.setCenter(SMap.Coords.fromWGS(ALatitude, ALongitude));
}

Geosetter.prototype.SetZoomLevel = function(ALevel)
{
  var nearestZoom = this.map.getNearestZoom({layerFrom: "GSatellite_ID", zoomFrom: ALevel});

  if (this.map.getZoom() != nearestZoom)
    this.map.setZoom(nearestZoom);
}

Geosetter._encapsulate("zoomBounds");
Geosetter._encapsulate("setCenter");
Geosetter._encapsulate("SetZoomLevel");

//********************************************************************************
//                              Blue Markers
//********************************************************************************

Geosetter.prototype._initBlueMarkers = function()
{
  this.markerBlueLayer = new SMap.Layer.Marker();
  this.map.addLayer(this.markerBlueLayer);
  this.markerBlueLayer.enable();
  
  var that = this;
 
  this.signals.addListener(window, "marker-drag-stop", function(e, elm)
  {
    var marker = e.target;
    var id = marker.getId();
    
    if (!id.match || !id.match(/blue/))   /* vyfiltrování nesouvisejících markerů */
      return;
      
    var AID = id.replace("blue2_", "").replace("blue3_", "");
    
    if (!that._didWeDrag(marker, AID))   /* kontrola, zda jsme táhli */
      return;    
    
    document.getElementById("current_marker").innerHTML = AID;

    if (id.match(/blue3_/))
    {
      var coords = marker.getCoords();
      document.getElementById("current_marker_lng").innerHTML = coords.lon();
      document.getElementById("current_marker_lat").innerHTML = coords.lat();
      
      // reposition blue marker on Google Maps
      for (var i = 0; i < Markers.length; i++)
      {
        if (Markers[i].ID == AID)
        {
          Markers[i].Marker.setPosition(coords.toGoogle());
          break;
        }
      }
    }
  });

  this.signals.addListener(window, "marker-click", function(e)
  {
    var marker = e.target;
    var id = marker.getId();
    
    if (!id.match || !id.match(/blue/))   /* vyfiltrování nesouvisejících markerů */
      return;
      
    var AID = id.replace("blue2_", "").replace("blue3_", "");
    
    if (id.match(/blue3_/))
    {
      document.getElementById("current_marker").innerHTML = AID;
    }
  });
}

Geosetter.prototype._getMarkerFromId = function(layer, id)
{
  var markers = layer._markers;
  
  if (id in markers)
    return markers[id].marker;
  else
    return null;
}

// někdy po kliknutí na marker vyskočí okénko nabízející změnu souřadnic
// -- to znamená, že uhodil opožděně marker-drag-stop
// -- v takovém případě bude posun nulový, drag tedy vrátíme
Geosetter.prototype._didWeDrag = function(marker, AID)
{
  var coordsAfterDrag = marker.getCoords();

  // projdeme všechny markery a najdeme původní souřadnice našeho markeru
  for (var i = 0; i < Markers.length; i++)
  {
    if (Markers[i].ID == AID)
    {
      var coordsBeforeDrag = SMap.Coords.fromGoogle(Markers[i].Marker.getPosition());
      var pixelDistance = coordsBeforeDrag.pixelDistance(this.map, coordsAfterDrag);
      
      // pokud jsme se pixelově neposunuli, tak obnovíme původní souřadnice markeru
      if (pixelDistance == 0)
      {
        marker.setCoords(coordsBeforeDrag);
        return false;
      }
      
      break;
    }
  }
  
  return true;
}

Geosetter.prototype._createBlueMarker = function(ALatitude, ALongitude, AID, ADraggable)
{
  var coords = new SMap.Coords.fromWGS(ALatitude, ALongitude);
  var options = {url: JAK.mel("img", {className: "gsBlueMarker", src: this.gsBasePath + "img/marker_blue.png"}), anchor: {left: 10, top: 34}};
  var marker = new SMap.Marker(coords, (ADraggable ? "blue3_" : "blue2_") + AID, options);

  if (ADraggable)
    marker.decorate(SMap.Marker.Feature.Draggable);

  this.markers[AID] = marker;

  this.markerBlueLayer.addMarker(marker);
}

Geosetter.prototype.addMarker2 = function(ALatitude, ALongitude, AID, ACaption, AThumbnailFile, ADate, AFocalLength, AAperture, AISO, AExposureTime, ACenter)
{
  this._createBlueMarker(ALatitude, ALongitude, AID, false);
}

Geosetter.prototype.addMarker3 = function(ALatitude, ALongitude, AID, ADirLat1, ADirLon1, ADirLat2, ADirLon2)
{
  var coords = SMap.Coords.fromWGS(ALatitude, ALongitude);

  // získáme marker
  var marker = this._getMarkerFromId(this.markerBlueLayer, "blue2_" + AID) || this._getMarkerFromId(this.markerBlueLayer, "blue3_" + AID);

  // hodnoty > 900 značí, že máme marker smazat
  if (ALatitude > 900 || ALongitude > 900)
  {
    if (marker)
      this.markerBlueLayer.removeMarker(marker);

    return;
  }
  
  // marker už existuje --> jen upravíme souřadnice
  if (marker)
  {
    marker.setCoords(coords);
  }
  
  // marker ještě neexistuje --> vytvoříme ho
  else
  {
    this._createBlueMarker(ALatitude, ALongitude, AID, true);
  }
}

Geosetter._encapsulate("addMarker2");
Geosetter._encapsulate("addMarker3");

//********************************************************************************
//                              Focus Marker
//********************************************************************************

Geosetter.prototype._initFocusMarker = function()
{ 
  this.focusMarkerLayer = new SMap.Layer.Marker("focusMarkerLayer");
  this.map.addLayer(this.focusMarkerLayer);
  this.focusMarkerLayer.enable();
  
  var that = this;

  this.signals.addListener(window, "marker-drag-stop", function(e, elm)
  {
    var marker = e.target;
    
    if (marker.getId() != "focused")
      return;
      
    if (!that._didWeDrag(marker, marker.AID))  /* kontrola, zda jsme táhli */
      return;

    var coords = marker.getCoords();
    document.getElementById("current_marker").innerHTML = marker.AID;
    document.getElementById("current_marker_lng").innerHTML = coords.lon();
    document.getElementById("current_marker_lat").innerHTML = coords.lat();
  });  
}  

Geosetter.prototype.showFocusMarker = function(ALatitude, ALongitude, AID, ACenter, APOVLat, APOVLon, APOVLat1, APOVLon1, APOVLat2, APOVLon2, AIsNewPOV)
{
  var coords = new SMap.Coords.fromWGS(ALatitude, ALongitude);

  this.focusMarkerLayer.removeAll();

  var options = {url: JAK.mel("img", {className: "gsFocusedMarker", src: this.gsBasePath + "img/marker_focused.png"}), anchor: {left: 10, top: 34}};
  this.markerFocused = new SMap.Marker(coords, "focused", options);
  this.markerFocused.decorate(SMap.Marker.Feature.Draggable);
  
  this.focusMarkerLayer.addMarker(this.markerFocused);
  
  // při kliku je třeba volat GeoSetter, aby poslal nový text karty
  JAK.Events.addListener(this.markerFocused.getContainer()[SMap.LAYER_MARKER], 'click', this, function()
  {
    document.getElementById("command").innerHTML = "marker_right_clicked," + AID;
  });  
  
  // set AID
  this.markerFocused.AID = AID;

  // center
  if(ACenter)
    this.map.setCenter(coords);
    
  // reposition blue marker
  if (AID && this.markers[AID])
    this.markers[AID].setCoords(coords);
    
  // nakreslíme POV trojúhelník
  this._drawPOVTriangle(AID, ALatitude, ALongitude, APOVLat1, APOVLon1, APOVLat2, APOVLon2, AIsNewPOV, true);
}

Geosetter._encapsulate("showFocusMarker");

//********************************************************************************
//                           Drag Marker (Red)
//********************************************************************************

Geosetter.prototype._initDragMarker = function()
{
  // drag marker layer
  this.dragMarkerLayer = new SMap.Layer.Marker("dragMarkerLayer");
  this.map.addLayer(this.dragMarkerLayer);
  this.dragMarkerLayer.enable();

  var options = {url: JAK.mel("img", {className: "gsDragMarker", src: this.gsBasePath + "img/marker_red.png"}), anchor: {left: 10, top: 34}};
  this.dragMarker = new SMap.Marker(new SMap.Pixel(0,0).toCoords(this.map), "dragMarker", options);
  this.dragMarker.decorate(SMap.Marker.Feature.Draggable);
  this.dragMarkerLayer.addMarker(this.dragMarker);
  
  this.markers[0 /*AID*/] = this.dragMarker;
  
  var that = this;
  
  // při kliku je třeba volat GeoSetter, aby poslal nový text karty
  JAK.Events.addListener(this.dragMarker.getContainer()[SMap.LAYER_MARKER], 'click', this, function()
  {
    document.getElementById("command").innerHTML = "marker_right_clicked,0";
  });
  
  this.signals.addListener(window, "marker-drag-stop", function(e, elm)
  {
    var marker = e.target;

    if (marker.getId() == "dragMarker")
    {
      that._moveDragMarker(e.target.getCoords());
    }
  });
  
  // klik na mapu --> posuneme tam Drag Marker
  this.signals.addListener(window, "map-click", function(e, elm)
  {
    // move drag marker on map click (but not on CTRL, which is for clicking the map within the track mode)
    if (!e.data.event.ctrlKey)
    {
      that._moveDragMarker(new SMap.Coords.fromEvent(e.data.event, that.map));
      that.setDragMarkerVisible(true);
    }
  });  
}

Geosetter.prototype._moveDragMarker = function(coords, AScrollInView)
{
  // v polygon módu marker nepřesouváme
  // -- bylo by dobré při CTRL povolit přesun, ale nefunguje to :(
  if (window.polygon && window.polygon.polygonModeOn /*&& !contains_class(this.map.getContainer(), "key_down_ctrl")*/)
    return;
    
  // skryjeme kartu, pokud je otevřená
  this.map.removeCard();

  this.dragMarker.setCoords(coords);
  document.getElementById("drag_mrk_lng").innerHTML = coords.lon();
  document.getElementById("drag_mrk_lat").innerHTML = coords.lat();
  
  if (AScrollInView)
    this.map.setCenter(coords);
    
  setDragMarkerToPositionOrig(coords.lat(), coords.lon(), false);
}

Geosetter.prototype.setDragMarkerVisible = function(AVisible)
{
  this.dragMarker.getContainer()[SMap.LAYER_MARKER].style.display = AVisible ? "" : "none";
}

Geosetter.prototype.gotoDragMarker = function()
{
  this.setDragMarkerVisible(true);
  
  var center = gDragMarker.getPosition();
  this.map.setCenter(SMap.Coords.fromGoogle(center));
}

Geosetter.prototype.setDragMarkerToPosition = function(ALatitude, ALongitude, AScrollInView)
{
  if (ALatitude > 999 || ALongitude > 999)
  {
    this.setDragMarkerVisible(false);
    return;
  }

  var coords = new SMap.Coords.fromWGS(ALatitude, ALongitude);
  this._moveDragMarker(coords, AScrollInView);
  this.setDragMarkerVisible(true);
}

Geosetter._encapsulate("setDragMarkerVisible");
Geosetter._encapsulate("gotoDragMarker");
Geosetter._encapsulate("setDragMarkerToPosition");

//********************************************************************************
//                                  Tracks
//********************************************************************************

Geosetter.prototype._initTrack = function()
{ 
  // track layer
  this.trackLayer = new SMap.Layer.Geometry("trackLayer");
  this.map.addLayer(this.trackLayer);
  this.trackLayer.enable();
}

Geosetter.prototype._createTrack = function(AID, AColor, AHint, AWidth, AOpacity)
{
  // pokud už náhodou track máme vytvořen, tak ho nejdříve smažeme
  this.removeTrack(AID);

  // najdeme body tracku a vytvoříme ho
  for (var i = 0; i < Tracks.length; i++)
  {
    if (Tracks[i].ID == AID) 
    { 
      // convert points
      var points = [];
      
      for (var j = 0; j < Tracks[i].Points.length; j++)
      {
        points.push(new SMap.Coords.fromGoogle(Tracks[i].Points[j]));
      }

      // create track
      var options = {color: AColor, width: AWidth, outlineWidth: 0, outlineColor: AColor, opacity: AOpacity};
      var track = new SMap.Geometry(SMap.GEOMETRY_POLYLINE, AID, points, options);
      this.trackLayer.addGeometry(track);
      
      break;
    }
  }
}

Geosetter.prototype.addTrack = function(AID, AColor, AHint)
{
  this._createTrack(AID, AColor, AHint, 10, 0.65);  
}

Geosetter.prototype.addTrack2 = function(AID, AColor, AHint, AWidth)
{
  this._createTrack(AID, AColor, AHint, AWidth, 0.65);  
}

Geosetter.prototype.addTrackWithOpacity = function(AID, AColor, AHint, AWidth, AOpacity)
{
  this._createTrack(AID, AColor, AHint, AWidth, AOpacity);
}

Geosetter.prototype.removeTrack = function(AID)
{
  var geometries = this.trackLayer.getGeometries();

  if (AID in geometries)
  {
    this.trackLayer.removeGeometry(geometries[AID]);
  }
}

Geosetter.prototype.showTrack = function(AID, AShow)
{
  var geometries = this.trackLayer.getGeometries();

  if (AID in geometries)
  {
    var track = geometries[AID];
    track.setOptions({opacity: AShow ? 0.65 : 0});
  }
}

Geosetter._encapsulate("addTrack");
Geosetter._encapsulate("addTrack2");
Geosetter._encapsulate("addTrackWithOpacity");
Geosetter._encapsulate("removeTrack");
Geosetter._encapsulate("showTrack");

//********************************************************************************
//                                    POV
//********************************************************************************

Geosetter.prototype._initPOV = function()
{
  // POV marker layer
  this.POVMarkerLayer = new SMap.Layer.Marker("POVMarkerLayer");
  this.map.addLayer(this.POVMarkerLayer);
  this.POVMarkerLayer.enable();

  // POV direction layer (focused violet)
  this.directionFocusedLayer = new SMap.Layer.Geometry("directionFocusedLayer");
  this.map.addLayer(this.directionFocusedLayer);
  this.directionFocusedLayer.enable();

  // POV direction layer (blue)
  this.directionLayer = new SMap.Layer.Geometry("directionLayer");
  this.map.addLayer(this.directionLayer);
  this.directionLayer.enable();

  // create marker POV New
  var point = SMap.Coords.fromWGS(0,0);
  var options = {url: JAK.mel("img", {className: "gsPOVMarkerNew", src: this.gsBasePath + "img/pov_new_mini.png"}), anchor: {left: 8, top: 8}};
  this.gsPOVMarkerNew = new SMap.Marker(point, "gsPOVMarkerNew", options);
  this.gsPOVMarkerNew.decorate(SMap.Marker.Feature.Draggable);
  this.POVMarkerLayer.addMarker(this.gsPOVMarkerNew);
  
  // create marker POV Assigned
  var options = {url: JAK.mel("img", {className: "gsPOVMarkerAssigned", src: this.gsBasePath + "img/pov_assigned_mini.png"}), anchor: {left: 8, top: 8}};
  this.gsPOVMarkerAssigned = new SMap.Marker(point, "gsPOVMarkerAssigned", options);
  this.gsPOVMarkerAssigned.decorate(SMap.Marker.Feature.Draggable);
  this.POVMarkerLayer.addMarker(this.gsPOVMarkerAssigned);  
  
  // updatování GeoSetteru po tažení POV markeru
  var that = this;

  this.signals.addListener(window, "marker-drag-stop marker-drag-move", function(e, elm)
  {
    var marker = e.target;
    var id = marker.getId();
    
    if (id == "gsPOVMarkerNew" || id == "gsPOVMarkerAssigned")
    {
      document.getElementById("pov_marker").innerHTML = that.gsPOVMarkerID;
      document.getElementById("pov_marker_lat").innerHTML = marker.getCoords().lat();
      document.getElementById("pov_marker_lng").innerHTML = marker.getCoords().lon();      
    }
  });
}

// odstraní modrý trojúhelník
Geosetter.prototype._removePOVTriangle = function(AID)
{
  var geometries = this.directionLayer.getGeometries();

  if (AID in geometries)
  {
    this.directionLayer.removeGeometry(geometries[AID]);
  }
}

// pro kreslení modrého i fialového trojúhelníku (fialový je pokud isActive = true)
Geosetter.prototype._drawPOVTriangle = function(AID, APOVLat, APOVLon, APOVLat1, APOVLon1, APOVLat2, APOVLon2, AIsNewPOV, isActive)
{
  if (!isActive)
    this._removePOVTriangle(AID);

  if (APOVLat1 == null || APOVLat1 >= 900)
    return;

  var p1 = SMap.Coords.fromWGS(APOVLat, APOVLon);
  var p2 = SMap.Coords.fromWGS(APOVLat1, APOVLon1);
  var p3 = SMap.Coords.fromWGS(APOVLat2, APOVLon2);
  
  // pokud máme aktivní POV, tak body p2 a p3 ukazují jen směr, získáme nové souřadnice protažením do dálky
  if (isActive)
  {
    var dist = p1.distance(AIsNewPOV ? this.gsPOVMarkerNew.getCoords() : this.gsPOVMarkerAssigned.getCoords());
    dist *= 1.5;
    p2 = p1.projectPoint(p1.azimuth(p2), dist);
    p3 = p1.projectPoint(p1.azimuth(p3), dist);
  } 

  var points = [p1, p2, p3];
  var color = isActive ? "#7700FF" : "#0000FF";
  var options = {color: color, outlineWidth: 0, outlineColor: color, outlineWidth: 2, opacity: 0.35};
  var geom = new SMap.Geometry(SMap.GEOMETRY_POLYGON, AID, points, options);

  // neaktivní --> šoupneme do directionLayer
  if (!isActive)
  {
    this.directionLayer.addGeometry(geom);
  }
  
  // aktivní --> šoupneme do directionFocusedLayer + odstraníme minulý aktivní POV
  else
  {
    if (this.gsPOVTriangle)
      this.directionFocusedLayer.removeGeometry(this.gsPOVTriangle);
  
    this.gsPOVTriangle = geom;
    this.directionFocusedLayer.addGeometry(this.gsPOVTriangle);
  }
}

Geosetter.prototype.showPOVMarker = function(ALocLat, ALocLon, ADestLat, ADestLon, AID, ACenter, AIsNew)
{
  // pomocná funkce, která nastaví viditelnost markeru
  function setMarkerVisibility(marker, trueOrFalse)
  {
    marker.getContainer()[SMap.LAYER_MARKER].style.display = trueOrFalse ? "" : "none";
  }
  
  // zavřeme kartu, pokud je otevřená
  this.map.removeCard();

  // zrušíme čáru k POV markeru (budeme kreslit novou)  
	if (this.gsPOVLine != null)
  {
    this.directionFocusedLayer.removeGeometry(this.gsPOVLine);
		this.gsPOVLine = null;
	}
  
  if (this.gsPOVTriangle && ADestLat > 900)
  {
    this.directionFocusedLayer.removeGeometry(this.gsPOVTriangle);
    this.gsPOVTriangle = null;
  }

	if (ADestLat > 900 || ADestLon > 900)
  {
		this.gsPOVMarkerNew.AID = -1;
		this.gsPOVMarkerAssigned.AID = -1;
		this.gsPOVMarkerID = -1;
    
    setMarkerVisibility(this.gsPOVMarkerNew, false);
    setMarkerVisibility(this.gsPOVMarkerAssigned, false);

    return;
  }

	if (AIsNew)
  {
		this.gsPOVMarkerNew.AID = AID;
		this.gsPOVMarkerID = AID;
		this.gsPOVMarkerNew.setCoords(SMap.Coords.fromWGS(ADestLat, ADestLon));
    
    setMarkerVisibility(this.gsPOVMarkerAssigned, false);
    setMarkerVisibility(this.gsPOVMarkerNew, true);
	}
  else
  {
		this.gsPOVMarkerAssigned.AID = AID;
		this.gsPOVMarkerID = AID;
		this.gsPOVMarkerAssigned.setCoords(SMap.Coords.fromWGS(ADestLat, ADestLon));

    setMarkerVisibility(this.gsPOVMarkerNew, false);
    setMarkerVisibility(this.gsPOVMarkerAssigned, true);
	}

  // create POV line
  var points = [SMap.Coords.fromWGS(ALocLat, ALocLon), SMap.Coords.fromWGS(ADestLat, ADestLon)];
  var options = {color: "#FF0000", width: 2, outlineWidth: 0, opacity: 1};
  this.gsPOVLine = new SMap.Geometry(SMap.GEOMETRY_POLYLINE, null, points, options);
  this.directionFocusedLayer.addGeometry(this.gsPOVLine);

	if (ACenter)
		this.map.setCenter(SMap.Coords.fromWGS(ALocLat, ALocLon));    
}

Geosetter.prototype.addDirection = function(AID, ALatitude, ALongitude, ADirLat1, ADirLon1,	ADirLat2, ADirLon2)
{                                                       
  if (this.markers[AID])
  {
    var coords = this.markers[AID].getCoords();
    this._drawPOVTriangle(AID, ALatitude, ALongitude, ADirLat1, ADirLon1, ADirLat2, ADirLon2); 
  }
}

Geosetter.prototype.removeDirection = function(AID)
{
  if (!this.markerFocused || this.markerFocused.AID != AID)
    this._removePOVTriangle(AID);
}

Geosetter.prototype.removeAllDirections = function()
{
	this.directionLayer.removeAll();
}

Geosetter.prototype.updateFocusMarkerDirection = function(AID, APOVLat1, APOVLon1, APOVLat2, APOVLon2)
{
	if (!this.markerFocused || this.markerFocused.AID != AID)
    return;
    
  var coords1 = this.markerFocused.getCoords();  
  this._drawPOVTriangle(AID, coords1.lat(), coords1.lon(), APOVLat1, APOVLon1, APOVLat2, APOVLon2, false, true);  
}

Geosetter._encapsulate("showPOVMarker");
Geosetter._encapsulate("addDirection");
Geosetter._encapsulate("removeDirection");
Geosetter._encapsulate("removeAllDirections");
Geosetter._encapsulate("updateFocusMarkerDirection");

//********************************************************************************
//                    karta, která se zobrazí při kliku na marker
//********************************************************************************

// existující problémy:
// -- klik u nás na červený marker, překlik do Google Maps --> prázdný hint

Geosetter.prototype._initMarkerCards = function()
{
  // zavřeme kartu u Google Maps při kliku na marker nebo ručním zavření karty
  // -- protože je synchronizace problematická
  this.signals.addListener(window, "marker-click card-close-click ", function(e)
  {
    gInfoWindow.close();
  });
  
  // zabráníme tomu, aby se po tažení drag markeru a focused markeru zobrazil hint
  var that = this;
  
  this.signals.addListener(window, "marker-drag-stop", function(e, elm)
  {
    var marker = e.target;
    var id = marker.getId();
    
    if (id == "dragMarker" || id == "focused")
      that.noShowHint = true;
  });  
}

// ukáže kartu (slouží pro funkci pod touto i pro Favorites)
Geosetter.prototype._showHint = function(marker, AText)
{
  if (!marker)
    return;
    
  var c = new SMap.Card();
  c.setSize(311, null); /* Šířka (default: 311px, null results in default value), výška */
  c.getBody().style.backgroundColor = 'white';
  c.getBody().innerHTML = AText;
  
  this.map.removeCard();
  this.map.addCard(c, marker.getCoords());
}

Geosetter.prototype.showMarkerHint2 = function(AID, AText)
{
  // po tažení hint drag markeru a focused markeru hint nezobrazujeme
  if (this.noShowHint)
  {
    this.noShowHint = false;
    return;
  }

  var marker = null;
  
  if (this.markerFocused && this.markerFocused.AID == AID)
    marker = this.markerFocused;
  else if (this.markers[AID])
    marker = this.markers[AID];

  this._showHint(marker, AText);
}

Geosetter._encapsulate("showMarkerHint2");

//********************************************************************************
//                     Favorites
//********************************************************************************

Geosetter.prototype._initFavorites = function()
{
  // favorites marker layer
  this.favoritesMarkerLayer = new SMap.Layer.Marker("favoritesMarkerLayer");
  this.map.addLayer(this.favoritesMarkerLayer);
  this.favoritesMarkerLayer.enable();
  
  // favorites circle layer
  this.favoritesCircleLayer = new SMap.Layer.Geometry("favoritesCircleLayer");
  this.map.addLayer(this.favoritesCircleLayer);
  this.favoritesCircleLayer.enable();
  
  var that = this;
  
  this.signals.addListener(window, "marker-drag-stop", function(e, elm)
  {
    var marker = e.target;
    var id = marker.getId();
    
    if (!id.match || !id.match(/fav_/))
      return;
      
    var AID = id.replace("fav_", "");
    var coords = marker.getCoords();

		document.getElementById("favorite_moved_id").innerHTML = AID;
		document.getElementById("favorite_moved_lat").innerHTML = coords.lat();
		document.getElementById("favorite_moved_lng").innerHTML = coords.lon();
    
    // je třeba ještě přesunout marker na druhé mapě
    that._syncWithGoogle(AID, false);
  });
  
  this.signals.addListener(window, "marker-click", function(e, elm)
  {
    var marker = e.target;
    var id = marker.getId();
    
    if (!id.match || !id.match(/fav_/))
      return;
      
    var AID = id.replace("fav_", "");

    document.getElementById("favorite_clicked").innerHTML = AID;
  }); 
}

// synchronizace aktuální Favorite mezi Google a naší mapou
// -- je třeba spustit, po každém tažení v Google Maps nebo u nás
Geosetter.prototype._syncWithGoogle = function(AID, fromGoogle)
{
  // marker z Favorites[]
  var FavMarker = null;

  for (var i = 0; i < Favorites.length; i++)
  {
    if (Favorites[i].ID == AID)
    {
      FavMarker = Favorites[i].Marker;
      break;
    }
  }

  // náš marker
  var marker = this._getMarkerFromId(this.favoritesMarkerLayer, "fav_" + AID);
  
  // pokud jsme nenašli (což bysme měli), tak ukončíme
  if (!marker || !FavMarker)
    return;

  // pokud synchronizujeme náš marker    
  if (fromGoogle)
  {
    marker.setCoords(SMap.Coords.fromGoogle(FavMarker.getPosition()));
  }
  // pokud synchronizujeme marker z Favorites[]
  else
  {
    FavMarker.setPosition(marker.getCoords().toGoogle());
  }
}

Geosetter.prototype.addFavorite = function(AID, ALatitude, ALongitude, AName)
{
  var coords = SMap.Coords.fromWGS(ALatitude, ALongitude);
  
  // pokusíme se získat marker
  var marker = this._getMarkerFromId(this.favoritesMarkerLayer, "fav_" + AID);
  
  // marker už existuje --> jen upravíme souřadnice
  if (marker)
  {
    marker.setCoords(coords);
    return;
  }

  // marker ještě neexistuje --> vytvoříme ho
  var options = {url: JAK.mel("img", {title: AName, className: "gsFavoritesMarker", src: this.gsBasePath + "img/favorite4.png"}), anchor: {left: 16, top: 16}};
  marker = new SMap.Marker(coords, "fav_" + AID, options);
  marker.decorate(SMap.Marker.Feature.Draggable);
  this.favoritesMarkerLayer.addMarker(marker);
  
  // nastavíme handler, který při dragendu v GoogleMaps posune i náš marker
  var that = this;

  for (var i = 0; i < Favorites.length; i++)
  {
    if (Favorites[i].ID == AID)
    {
      google.maps.event.addListener(Favorites[i].Marker, 'dragend',  function()
      {
        that._syncWithGoogle(AID, true);
      });
    }
  }  
}

Geosetter.prototype.removeFavorites = function(AID)
{
  this.favoritesMarkerLayer.removeAll();
}
 
Geosetter.prototype.removeFavoriteCircles = function()
{
  this.favoritesCircleLayer.removeAll();
}

Geosetter.prototype.addFavoriteCircle2 = function(AID, ALatitude, ALongitude, ARadius, AColor)
{
  // kruh už existuje --> ukončíme
  var circles = this.favoritesCircleLayer.getGeometries();

  if (AID in circles)
    return;
  
  // kruh ještě neexistuje --> vytvoříme ho
  var center = SMap.Coords.fromWGS(ALatitude, ALongitude);
  var point_on_circle = center.projectPoint(0, ARadius);
  AColor = "black";
  var options = {color: AColor, opacity: 0.3, outlineWidth: 3, outlineColor: AColor, outlineOpacity: 0.7};
  var circle = new SMap.Geometry(SMap.GEOMETRY_CIRCLE, AID, [center, point_on_circle], options);
  this.favoritesCircleLayer.addGeometry(circle);
}

Geosetter.prototype.showFavoriteHint = function(AID, AText)
{
  var marker = this._getMarkerFromId(this.favoritesMarkerLayer, "fav_" + AID);
  this._showHint(marker, AText);
}

Geosetter._encapsulate("addFavorite");
Geosetter._encapsulate("removeFavorites");
Geosetter._encapsulate("removeFavoriteCircles");
Geosetter._encapsulate("addFavoriteCircle2");
Geosetter._encapsulate("showFavoriteHint");

//********************************************************************************
//                                Crosshair
//********************************************************************************

Geosetter.prototype._initCrosshair = function()
{ 
  // skryjeme původní kříž
  document.getElementById("crosshair").style.display = "none";
  
  // u IE <= 8 je možno obrázek táhnout, což má za následek vybrání dokumentu
  // -- takže zrušíme vybrání při každém překreslení mapy
  // -- bylo by dobré najít lepší způsob 
  if (JAK.Browser.client == "ie" && JAK.Browser.version <= 8)
  {
    this.signals.addListener(window, "map-redraw", function(e, elm)
    {
      document.selection.empty();
    });
  }  
}

Geosetter.prototype.showCenterCross = function(AShow)
{
  var mapContainer = this.map.getContainer();
  var mapContainerGoogle = map.getDiv();
  
  // zobrazíme kříž
  if (AShow)
  {
    add_class(mapContainer, "gsCrosshair");
    add_class(mapContainerGoogle, "gsCrosshair");
  }
  
  // skryjeme kříž
  else
  {
    remove_class(mapContainer, "gsCrosshair");
    remove_class(mapContainerGoogle, "gsCrosshair");
  }
}

Geosetter._encapsulate("showCenterCross");

//********************************************************************************
//                           Custom Layers (TO DO)
//********************************************************************************

Geosetter.prototype._initCustomLayers = function()
{ 

}

//********************************************************************************

//if (window.isGeoSetter && JAK.Browser.embedded)   /* bylo by lepší, ale kdyby to náhodou špatně detekovalo, tak aplikace v GeoSetteru nebude vůbec fungovat */
if (window.isGeoSetter && JAK.Browser.client == "ie")
  var gs = new Geosetter(m);

//log("load_end");