//*******************************************************************
//                     Search - construct class
//*******************************************************************

SMap.Control.Search = JAK.ClassMaker.makeClass({
	NAME: "SMap.Control.Search",
	VERSION: "1.0",
	EXTEND: SMap.Control.Visible
});

SMap.Control.Search.prototype.$constructor = function(options) {
	this.$super();

	this._options = {}
	for (var p in options) { this._options[p] = options[p]; }
	
	this._build();
}

SMap.Control.Search.prototype.$destructor = function()
{
  // http://api.mapy.cz/js/api/api/control.js
	this.getMap().getSignals().removeListeners(this._sc);
  JAK.Events.removeListeners(this._ec);
	this.$super();
}

/**
 * @see SMap.Control#setOwner
 */
SMap.Control.Search.prototype.setOwner = function(owner) {
	this.$super(owner);
	if (!owner) { return; }
}

//*******************************************************************
//           Search - create form, handle focus + exit, ...
//*******************************************************************

SMap.Control.Search.prototype._build = function()
{
  /* create main DOM elements */
  this._dom.container = JAK.mel("div", {className: "search-form"});
  this._dom.input = JAK.mel("input", {className: "search-input", type: "text", autocomplete: "off"});
  this._dom.results = JAK.mel("div", {className: "search-results"});
  
  JAK.DOM.append([this._dom.container, this._dom.input, this._dom.results]);
  
  /* create anchors for different groups of search results */
  this._dom.resultsCoords = JAK.mel("span", {className: "search-results-coords"});
  this._dom.resultsSeznam = JAK.mel("span", {className: "search-results-seznam"});
  this._dom.resultsGeoNames = JAK.mel("span", {className: "search-results-geonames"});
  this._dom.resultsCustom = JAK.mel("span", {className: "search-results-custom"}); /* caches */
  
  this._anchors = [this._dom.resultsCoords, this._dom.resultsSeznam, this._dom.resultsGeoNames, this._dom.resultsCustom];
  JAK.DOM.append([this._dom.results].concat(this._anchors));

  /* add input handlers */
  // https://developer.mozilla.org/en-US/docs/Web/Events/input
  this._ec.push(JAK.Events.addListener(this._dom.input, "keydown keyup keypress", this, "_preventKeyBubbling" /* and search on enter */));  
  this._ec.push(JAK.Events.addListener(this._dom.input, "focus", this, "_focusForm"));                        /* when we activate input */
  this._ec.push(JAK.Events.addListener(window, "resize", this, "_exitForm"));                                 /* exit on window resize */
  
  //*******************
  // přidáme handler, který opožďuje volání, pokud je moc volání hned po sobě
  // -- abychom příliš neprotěžovali server

  this._sc.push(JAK.Events.addListener(this._dom.input, "keyup input keypress", this, function(e, target)  /* keypress is for IE + backspace*/
  {
    // we delay checking input change
    // -- if there is input change already waiting, cancel it

    // if timeout already set then cancel it (so that we don't search old query)
    if (this._checkQueryChange_Timeout)
      window.clearTimeout(this._checkQueryChange_Timeout);

    // ... and set new timeout
    var that = this;

    this._checkQueryChange_Timeout = window.setTimeout(function()
    {
      that._checkQueryChange(e, target);   
    }, this._checkQueryChange_Timeout ? 200 : 0 /* run immediately when no timeout present */);
  }));
  
  // verze bez timeoutu:
  // this._ec.push(JAK.Events.addListener(this._dom.input, "keyup input keypress", this, "_checkQueryChange"));  /* keypress is for IE + backspace*/
  //*******************

  /* initialize query to empty string */
  this._query = "";
}

/*** launches when we click on the input ***/

SMap.Control.Search.prototype._focusForm = function(e, target)
{
  // set input to active
  if (this._inputActive)
    return;

  this._inputActive = true;

  // set handler that hit when we click outside of the form
  this._unfocusHandler = JAK.Events.addListener(document.documentElement, "click", this, "_unfocusForm");
    
  // select text, so we can overwrite it by just typing something new
  this._dom.input.select();
  
  // set width of input box (restrict it if there is not enough space)
  // -- maybe can be done using CSS
  var inputLeftPos = this._dom.container.offsetLeft;
  var mapRightPos = this.getMap().getSize().x;
  var availablePixels = mapRightPos - inputLeftPos - 160 /* allow for some items on the right */;
  var width = Math.max(90, Math.min(500, availablePixels));
  this._dom.container.style.width = width + "px";
  
  // set z-index so that it's on top everything else (highet than values in mapycz_mod.css)
  this._dom.container.style.zIndex = "1001";

  // fix some things in results div
  this._handleOverflow();     /* remove those results that overflow */
  this._handleRoundCorners(); /* and if we removed anything, then fix round corners too */
  
  // make event
  this.getMap().getSignals().makeEvent("search-field-start", this);
}

/*** launches when we click outside of the input ***/

SMap.Control.Search.prototype._unfocusForm = function(e, target)
{
  // we can (uncommonly) have already exited, which happens when we manage to press enter multiple times 
  if (!this._unfocusHandler)
    return;

  // return if we clicked on our input
  if (EventUtil.getTarget(e) === this._dom.input)
    return;

  this._exitForm();
}

/*** close down form ***/

SMap.Control.Search.prototype._exitForm = function()
{
  // make sure that we don't exit multiple times
  if (!this._inputActive)
    return;

  this._inputActive = false;

  this._dom.input.blur();    /* Firefox doesn't blur when clicking outside of input */

  // deselect input box
  // -- long selected text would otherwise cause alignment issues in Opera 12
  // -- sometimes it still doesn't work on IE8 
  var temptext = this._dom.input.value;
  this._dom.input.value = "";
  this._dom.input.value = temptext;
  
  // remove width style (so that CSS is used instead)
  this._dom.container.style.width = "";
  
  // remove z-index (so that it's low)
  this._dom.container.style.zIndex = "";
     
  // remove unfocus handler
  JAK.Events.removeListener(this._unfocusHandler);
  this._unfocusHandler = null;
  
  // make event
  this.getMap().getSignals().makeEvent("search-field-exit", this);    
}

/* zabránění, aby vložené znaky byly přenášeny nahoru do mapy */

SMap.Control.Search.prototype._preventKeyBubbling = function(e)
{
  // prevent key bubbling
  EventUtil.stopPropagation(e);
  
  //log(e.keyCode + ": " + e.type);

  // prevent submit (ENTER) and go to first search result (if any)
  // -- restricted to keydown to prevent another launch on keyup
  if (e.type == "keydown" && e.keyCode == 13)
  {
    EventUtil.preventDefault(e);
    this.enterAndExit();
  }
  
  // exit on pressing ESCAPE
  if (e.keyCode == 27)
  {
    this._exitForm({});
  }
}

// handle exit after pressing enter (go to first node)
SMap.Control.Search.prototype.enterAndExit = function()
{
  var that = this;

  // set timeout, so there is time to get results, after getting them proceed
  var totalTime = 0;
  var intervalLength = 400;
  
  var interval = window.setInterval(function()
  {
    // run this setInterval only max specified number of times
    totalTime += intervalLength;
    
    if (totalTime >= 3 * intervalLength) /* on 4th run, we stop searching and we don't exit if nothing found on this last run */
      window.clearInterval(interval);
              
    // get first result
    // -- to exclude a class we could use: 'not(.search-results-custom) .search-result' (doesn't work in IE7-8)
    var results = that._dom.results.querySelectorAll('.search-result');

    if (!results.length)
      return;
    
    // if there is a result, go there and exit
    that._goToPlace(results[0]);
    that._exitForm({});
    
    // clear the interval
    window.clearInterval(interval);
    
  }, intervalLength);
}

//*******************************************************************
//               Search - handle search results
//*******************************************************************

/* provede hledání (pokud nastala změna v řetězci) */

SMap.Control.Search.prototype._checkQueryChange = function(e, target)
{
  // get query from input field and set it to lowercase
  if (this._query.trim() === this._dom.input.value.trim())
    return;
  
  this._query = this._dom.input.value.toLocaleLowerCase().trim();

  // clear all existing results
  this._clearResults();

  if (this._query.length < 3)
    return;
  
  var that = this;

  /* try to parse coordinates */
  this.parseCoordinates();

  /* Seznam */
  this.searchSeznam();
  
  /* GeoNames */
  this.searchGeonames();  

  /* custom search (caches) */
  this.searchCustom();    
}

/* vyznačí ve výsledcích tučně hledaný řetězec */

SMap.Control.Search.prototype._boldQuery = function(str)
{
  // function that will escape query (because it will be used inside of regex)
  function escapeRegex(s)
  {
    return s.replace(/[-\/\\^$*+?.()|[\]{}]/g, '\\$&');
  };

  // replace all instances of query with <b>query</b>
  var re = new RegExp('(^|)(' + escapeRegex(this._query) + ')(|$)','ig');
  str = str.replace(re, '$1<b>$2</b>$3');

  return str;
}

/* smaže výsledky z okna výsledků */

SMap.Control.Search.prototype._clearResults = function(anchor)
{
  // if no anchor (Seznam, Geonames, coords, ...) specified as param, then clear ALL search results
  if (!anchor)
  {
    for(i = 0; i < this._anchors.length; i++)
       this._clearResults(this._anchors[i]);

    return;
  }
  
  // delete all search results of a given anchor
  while (anchor.firstChild)
  {
      anchor.removeChild(anchor.firstChild);
  }
}

/* spočte počet aktuálně zobrazených výsledků */
SMap.Control.Search.prototype._countResults = function()
{
  var results = this._dom.results.querySelectorAll('.search-result');

  return results.length;
}

SMap.Control.Search.prototype._addToResults = function(html, coords, anchor)
{
  // create div
  var div = JAK.mel("div");
  div.innerHTML = html;
  anchor.appendChild(div);

  // save coordinates to data attibutes
  div.setAttribute('data-searchresult-lat', coords.lat());
  div.setAttribute('data-searchresult-lon', coords.lon());
  JAK.DOM.addClass(div, 'search-result');
  
  // use mousedown here, because click would trigger _unfocusForm before launching this function
  var that = this;
   
  JAK.Events.addListener(div, "mousedown", this, function()
  {
    that._goToPlace(div);
    that._exitForm();
  })

  // remove results that overflow map viewport
  this._handleOverflow();  

  // mark first and last divs, which will have round corners
  this._handleRoundCorners();
}

// remove results that overflow map viewport
SMap.Control.Search.prototype._handleOverflow = function()
{
  var results = this._dom.results.querySelectorAll('.search-result');
  
  for (var i = 0; i < results.length; i++)
  {
    var mapBottom = this.getMap().getSize().y;
    var resultBottom = results[i].offsetTop + results[i].offsetHeight;
    
    if (mapBottom - resultBottom < 34)
      results[i].parentNode.removeChild(results[i]);
  }
}

// mark first and last divs, which will have round corners
SMap.Control.Search.prototype._handleRoundCorners = function()
{
  if (JAK.Browser.client == "ie" && JAK.Browser.version <= 8)
    return;

  // unmark previous first child
  var results = this._dom.results.querySelectorAll('.search-result.first-child');
  
  for (var i = 0; i < results.length; i++)
    JAK.DOM.removeClass(results[i], 'first-child');

  // unmark previous last child
  results = this._dom.results.querySelectorAll('.search-result.last-child');

  for (var i = 0; i < results.length; i++)
    JAK.DOM.removeClass(results[i], 'last-child');

  // mark current first child  
  results = this._dom.results.querySelectorAll('.search-result:first-child');
  
  if (results.length)
    JAK.DOM.addClass(results[0], 'first-child');

  // mark current last child    
  results = this._dom.results.querySelectorAll('.search-result:last-child');
  
  if (results.length)
    JAK.DOM.addClass(results[results.length - 1], 'last-child'); 
}

// after clicking on search result, go to that place
SMap.Control.Search.prototype._goToPlace = function(div)
{
  var coords = SMap.Coords.fromWGS(div.getAttribute('data-searchresult-lat'), div.getAttribute('data-searchresult-lon'));
  
  // switch to Google if the map is outside its bBox
  if (!coords.isValid(this.getMap()))
    this.getMap().switchMap("GSatellite_ID");  

  this.getMap().setCenter(coords);
  
  // create marker
  var markerSearchLayer = new SMap.Layer.Marker();
  this.getMap().addLayer(markerSearchLayer);
  markerSearchLayer.enable();
  var marker = new SMap.Marker(coords);
  markerSearchLayer.addMarker(marker);
  add_class(marker.getContainer()[SMap.LAYER_MARKER], "search-marker");
  
  // fade out the marker and destroy it after that
  var percentStart = 100;
  var percentEnd = 20;
  var intervalLength = 150; /* ms */
  
  var interval = window.setInterval(function()
  {
    marker.getContainer()[SMap.LAYER_MARKER].style.opacity = percentStart / 100;
    marker.getContainer()[SMap.LAYER_MARKER].style.filter = "alpha(opacity=" + percentStart + ")"; // IE
    
    // clear the interval and destroy marker
    if (percentStart-- < percentEnd)
    {
      window.clearInterval(interval);
      markerSearchLayer.disable();
      marker.getMap().removeLayer(markerSearchLayer);
    }
    
  }, intervalLength);
}

//*******************************************************************
//                     search Coordinates
//*******************************************************************

SMap.Control.Search.prototype.parseCoordinates = function()
{
  this._clearResults(this._dom.resultsCoords);

  var coords = SMap.Coords.fromString(this._query);
  
  if (!coords)
    return;

  var image = "<img src='" + window.basePath + "Images/Search/Coordinates.png'>";
  var label = "<b>" + coords.format("WGS", "min") + "</b>"

  this._addToResults(image + " " + label, coords, this._dom.resultsCoords);  
}

//*******************************************************************
//                     search Seznam
//*******************************************************************

SMap.Control.Search.prototype.searchSeznam = function()
{
  // doesn't work in IE7-8
  if (JAK.Browser.client == "ie" && JAK.Browser.version <= 8)
    return;

  new SMap.Geocoder(this._query, this.parseSeznamCallback, {url: "geocode.php", that: this, time: new Date().getTime()});
}

SMap.Control.Search.prototype.parseSeznamCallback = function(results)
{
  if (!results || !this.getResults()[0])
    return;

  var vysledky = this.getResults()[0].results;
  var that = this._options.that;
  
  // zkontrolujem, zda jsme nedostali výsledky z volání, které bylo voláno dříve než to pro zobrazené výsledky
  var lastTime = that._lastSeznamSearchTime || 0;
  var currentTime = this._options.time;
  
  if (currentTime < lastTime)
    return;
  
  that._lastSeznamSearchTime = currentTime;

  // vyčistíme staré výsledky
  that._clearResults(that._dom.resultsSeznam);

  for (i = 0; i < Math.min(vysledky.length, 10); i++) 
  {
    // string
    // id: 12705329
    // source: osmp
    // label: Restaurace Praha - Das kleine Prager Restaurant
    // coords: (131598214,139621088)  

    var image = "<img src='" + window.basePath + "Images/Search/Seznam.png'>";
    var label = that._boldQuery(vysledky[i].label);

    that._addToResults(image + " " + label, vysledky[i].coords, that._dom.resultsSeznam);
  }
}

//*******************************************************************
//                     search GeoNames
//*******************************************************************

SMap.Control.Search.prototype.searchGeonames = function()
{
  var time = new Date().getTime();
  var that = this;

  // *** funkce, ve které zjistíme, zda jsme nedostali starší výsledky než aktuálně zobrazené ***
  // -- volá ji callback javascriptu (a z této funkce pak voláme samotné parsování výsledků)
  window["geonamesCheck" + time] = function(jData)
  {
    // smažeme referenci na tuto funkci
    try {delete window["geonamesCheck" + time]} catch(e){window["geonamesCheck" + time] = undefined};  // IE8 neumí delete :(

    // zjistíme čas odeslání předchozí hledání a tohoto hledání
    var lastTime = that._lastGeonamesSearchTime || 0;
    var currentTime = time;

    // pokud toto hledání bylo spuštěno před předchozím hledáním, ukončíme
    if (currentTime < lastTime)
      return;

    // nastavíme čas pro porovnání, až budeme tuto funkci volat znovu
    that._lastGeonamesSearchTime = currentTime;
    
    // pokud jsme nedostali výsledky z předchozího hledání, zpracujeme je  
    that.parseGeonamesResults(jData);
  }

  // odešleme vyhledávání
  // reference: http://www.geonames.org/export/geonames-search.html
  // co pošle při překročení limitu: http://www.geonames.org/export/webservice-exception.html
  loadJS('http://api.geonames.org/searchJSON?q=' + encodeURIComponent(this._query) 
    + '&maxRows=25&username=gsakmapymakro&callback=geonamesCheck' + time);
}

SMap.Control.Search.prototype.parseGeonamesResults = function(jData)
{
  // smažeme staré výsledky
  this._clearResults(this._dom.resultsGeoNames);

  // zjistíme, zda nám to něco našlo  
  if (jData == null || !jData.totalResultsCount)
    return;

  var vysledky = jData.geonames;

  // projdeme výsledky a přidáme
  for (var i = 0; i < Math.min(vysledky.length, 10); i++)
  {
    var gn = vysledky[i];
    
    // population: 0
    // adminName1: 
    // lat: 50.55519
    // geonameId: 3070686
    // fcode: MT
    // name: Milešovka
    // fcl: T
    // toponymName: Milešovka
    // fcodeName: mountain
    // lng: 13.93162
    // countryCode: CZ
    // fclName: mountain,hill,rock,... 
    // countryName: Czech Republic
    // adminCode1: 00
    // countryId: 3077311
    
    // population: 7734614
    // adminName1: Muḩāfaz̧at al Qāhirah
    // lat: 30.06263
    // geonameId: 360630
    // fcode: PPLC
    // name: Cairo
    // fcl: P
    // toponymName: Cairo
    // fcodeName: capital of a political entity
    // lng: 31.24967
    // countryCode: EG
    // fclName: city, village,...
    // countryName: Egypt
    // adminCode1: 11
    // countryId: 357994

    var image = "<img src='" + window.basePath + "Images/Search/GeoNames.png'>";
    var label = this._boldQuery(gn.name) + ", " + gn.fcodeName + ", " + gn.countryName;
    var coords = SMap.Coords.fromWGS(gn.lat, gn.lng);

    this._addToResults(image + " " + label, coords, this._dom.resultsGeoNames);
  }
}

//*******************************************************************
//                     search Custom (Caches)
//*******************************************************************

SMap.Control.Search.prototype.searchCustom = function()
{
  this._clearResults(this._dom.resultsCustom);
  var total = 0;
  
  for (var code in wp)
  {
    var cache = wp[code];
    var name = cache.Name;
    
    if ((code.toLocaleLowerCase().indexOf(this._query) != -1)   /* je hledaný řetězec součástí kódu, např. GCXXXXX? */
    || (name.toLocaleLowerCase().indexOf(this._query) != -1))   /* je hledaný řetězec součástí názvu keše? */
    {
      if (total++ >= 60)
        break;    
    
      var coords = SMap.Coords.fromWGS(cache.Latitude, cache.Longitude);
      var label = code + ", " + name;
      var image = "<img src='";

      // Waymarks
      if (cache.CacheType == "Y")
      {
        image += cache.WaymarkCategoryIcon + "' width=16 height=16>";
      }
      // Cache parents
      else if (!cache.Parent)
      {
        image += window.basePath + "Images/Caches/" + cache.CacheType + ".png' width=16 height=16>";
        label += " (" + cache.Difficulty + "/" + cache.Terrain + ", " + cache.Container + ")";
      }
      // Cache children
      else if (cache.Parent)
      {
        image += window.basePath + "Images/Markers/" + cache.CacheType + ".png' width=10 height=16>";
        label += " (" + wp[cache.Parent].Name + ")";
      }

      label = this._boldQuery(label);
  
      this._addToResults(image + " " + label, coords, this._dom.resultsCustom);
    }
  }
}

//*******************************************************************
//                     context menu search
//*******************************************************************

//==============================================
//============== parse results =================
//==============================================

function parseLocationSearchResult(jData)
{
  if (jData == null || !jData.totalResultsCount)
  {
    ui("copybox", {text: tr("Nothing found!"), width: 200, height: 20});
    return;
  }

  // go through all received coords and retrieve closest ones
  var min_distance;
  var new_coords;

  for (var i = 0; i < jData.geonames.length; i++)
  {
    var g = jData.geonames[i];
    /* 
    // skip administrative divisions and country name != Czechia/Slovakia
    if (i > 0 && (g.fcodeName.match(/administrative division/) || !(g.countryName.match(/Czechia|Slovakia/))))
      continue;                        

    // get distance between received coords and our original coords
    var distance = geoname_lookup_coords.coords.distance(SMap.Coords.fromWGS(g.lat, g.lng)) / 1000;
    
    // set new coords if distance is lower then the previous one
    if (!new_coords || distance < min_distance)
    {
      new_coords = g;
      min_distance = distance;
    }
    */
    
    new_coords = g;
  }

  new_coords = SMap.Coords.fromWGS(new_coords.lat, new_coords.lng);

  // switch to Google if the map is outside its bBox
  if (!new_coords.isValid(m))
    m.switchMap("GSatellite_ID");

  // set new center
  m.setCenter(new_coords);
}

//==============================================
//============ GeoNames Geocoder ===============
//==============================================

function geonamesCallback(jData)
{ 
  parseLocationSearchResult(jData);
}

function geonamesGeocoder(location_name)
{
  // info: http://www.geonames.org/export/geonames-search.html
  //request = 'http://api.geonames.org/searchJSON?formatted=true&name_startsWith=' + encodeURIComponent(location_name) + '&maxRows=1&username=gsakmapymakro&style=full&countryBias=CZ&lang=cs&callback=geonamesCallback'
  loadJS('http://api.geonames.org/searchJSON?name=' + encodeURIComponent(location_name) + '&maxRows=100&countryBias=CZ&username=gsakmapymakro&callback=geonamesCallback');
}

//==============================================
//============== Seznam Geocoder ===============
//==============================================

function seznamCallback (result) 
{
  // received search results
  var vysledky = result.getResults()[0].results || 0;

  // declare our new array (that we send to GeoName processing function)
  var jData = {geonames: []};
   
  // go through array and convert all items to GeoName compatible format
  //for (i = vysledky.length - 1; i >= 0; i--)
  for (i = 0; i < vysledky.length; i++) 
  {
    var item = vysledky[i];
    item.lat = item.coords.lat();
    item.lng = item.coords.lon();
    item.fcodeName = "";
    item.countryName = ((item.label).match(/(esk..republika)/)) ? "Czechia" 
      : (((item.label).match(/Slovensko/)) ? "Slovakia" : "");
      
    // only CZ and SK - use geoname for everything else
    /*if (item.countryName != "")*/ jData.geonames.push(item);
    
    //log_object(vysledky[i]);
    break;
  }
  
  jData.totalResultsCount = jData.geonames.length;

  // if we have results, parse them 
  if (jData.geonames.length)
    parseLocationSearchResult(jData);
    
  // no results? run GeoNames
  else
    geonamesGeocoder(geoname_lookup_coords.location);
}

function seznamGeocoder (location_name) 
{
  return(new SMap.Geocoder(location_name, seznamCallback, {url:"geocode.php"}))
}

//==============================================
//============== main function =================
//==============================================

function LocationLookup(location_name, lat, lon)
{ 
  // we can't pass parameters to the callback, so send them through global variable
  window.geoname_lookup_coords = {location: location_name, coords: new SMap.Coords.fromWGS(lat || 50, lon || 15)};
  
  try
  {
    // SMap Geocoder fails in IE8 and older (because of xmlHttpRequest)
    new SMap.Geocoder(location_name, seznamCallback, {url:"geocode.php"});
  }
  catch(ex)
  {
    // if seznam fails, use GeoNames
    geonamesGeocoder(location_name);
  }  
}


//*******************************************************************
//                     launcher
//*******************************************************************

function load_search_field()
{
  window.search_field = null;
  
  var searchControl = new SMap.Control.Search();
  m.addControl(searchControl, {left:"150px",top:"10px"});
  
  window.search_field = searchControl._dom.container; 
}

//*******************************************************************
//                     context menu function
//*******************************************************************

function searchContextMenuHandler(target, menu, coords)
{
  var form = document.getElementById("findForm");

  // function, that executes online search
  window.findExecute = function ()
  {
    var place = form.elements["findBox"].value;
    
    if (place)
      LocationLookup(place, coords.lat(), coords.lon());
  }

  // initialize listeners
  if (!form.getAttribute("initialized"))
  {
    form.setAttribute("initialized", true);

    // prevent automatic submit (handle ENTER manually instead - automatic submit doesn't work in IE)
    EventUtil.addHandler(form, 'submit', function (e, target)
    {
      EventUtil.preventDefault(e);
    });

    // on ENTER execute search
    EventUtil.addHandler(form.elements["findBox"], 'keydown', function (e, target)
    {
      if (e.keyCode == 13)
      {
        window.findExecute();
        document.getElementById("findBox").blur();    /* fixes Opera 12.17 problem - findBox would be active even after closing the menu */
        EventUtil.preventDefault(e);
        menu.style.display = "none";
      }
    });
  }

  // execute search on clicking label, return (without closing menu) otherwise
  if (target.tagName.toLowerCase() == "span")
    window.findExecute();
  else
    return true;
}